#	  This file is part of Dim, a Digital Image Management program.
#	  Copyright 2003, Anssi Piirainen
#    Dim/Library.py - a domain model for an image library containing
#    catalogs and an index for fast retrieval of images
#
#    Dim is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Dim is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Dim; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
from ImageLib import *
from EXIFView import *

from UserDict import UserDict
import os
from stat import *
import Image
import gtk

def isDirectory(fileNameWithPath):
	mode = os.stat(fileNameWithPath)[ST_MODE]
	return S_ISDIR(mode)

class NoImageException(Exception):
	def __init__(self, value):
		self.value = value
	def __str__(self):
		return repr(self.value)

class DimImage:

	def __init__(self, fileName, thumbDir, checkImage=True):
		if checkImage:
			if isImage(fileName) == False: raise NoImageException("Image not found from file " + fileName)
		self.__fileName = fileName
		self.__thumbnailFactory = ThumbnailFactory(thumbDir)
	
	def hasThumb(self):
		return self.__thumbnailFactory.findThumbnail(self.__fileName) != None
	
	def getThumb(self):
		return self.__thumbnailFactory.findThumbnail(self.__fileName)
	
	def generateThumb(self):
		createThumbnail(self.__fileName, self.__thumbnailFactory.getThumbDir())
	
	def getEXIF(self):
		view = EXIFView(self.__fileName)
		if view == None: return
		return view.getExifFormatted()
	
	def getFileName(self):
		return self.__fileName
	
	def getFileBasename(self):
		return os.path.basename(self.__fileName)
	
	def show(self, viewImage, targetAllocation):
		viewImage.set_from_pixbuf(None)
		viewImage.set_from_pixbuf(loadAndScalePixbuf(self.getFileName(), targetAllocation.width - 20, targetAllocation.height - 20))
		viewImage.show()

class Catalog:
	
	def __init__(self, name):
		self.__images__ = []
		self.name = name
	
	def add(self, image):
		if self.hasImage(image): return
		self.__images__.append(image)
	
	def remove(self, image):
		if self.hasImage(image): self.__images__.remove(image)
	
	def getImages(self):
		return [f for f in self.__images__]
	
	def hasImageWithFileName(self, fileName):
		return fileName in [im.getFileName() for im in self.__images__]
	
	def hasImage(self, image):
		return image in self.__images__

class Library:
	
	def __init__(self):
		self.__catalogs = []
		self.__catalogsByName = { }
	
	def addCatalog(self, catalogName):
		catalog = self.getCatalog(catalogName)
		if catalog != None: return catalog
		
		catalog = Catalog(catalogName)
		self.__catalogs.append(catalog)
		self.__catalogsByName[catalogName] = catalog
		print "added catalog", catalog.name
		return catalog
	
	def removeCatalog(self, catalog):
		if catalog == None: return
		self.__catalogs.remove(catalog)
		del self.__catalogsByName[catalog.name]
	
	def getCatalog(self, catalogName):
		return self.__catalogsByName.has_key(catalogName) and self.__catalogsByName[catalogName] or None

	def hasCatalog(self, catalog):
		return catalog in self.__catalogs

	def hasCatalogWithName(self, catalogName):
		return getCatalog(catalogName) != None
	
	def getCatalogs(self):
		return [cat for cat in self.__catalogs]

class Index(UserDict):

	def __init__(self, thumbdir):
		UserDict.__init__(self)
		self.__thumbdir = thumbdir	
		self.__imageListener = None
		self.__imageListenerObject = None

	def getFolderImages(self, folder):
		if self.has_key(folder): return self[folder]
		
	def setImageListener(self, listener, listenerObject = None):
		self.__imageListener = listener
		self.__imageListenerObject = listenerObject
	
	def createFolderImages(self, folder):
		if folder == None: return
		try:
			files = os.listdir(folder)
		except OSError:
			return None
		self[folder] = [im for im in map(lambda f: (self.__createImage(f, folder)), files) if im != None]
		print "created a list", self[folder]
		
		if self.__imageListener:
			self.__imageListener.onImageFolderLoaded(self[folder], self.__imageListenerObject)
		return self[folder]

	def __createImage(self, fileName, folder):
		fileNameWithPath = os.path.join(folder, fileName)
		#if isDirectory(fileNameWithPath): return None
		image = None
		try:
			image = DimImage(fileNameWithPath, self.__thumbdir)
		except NoImageException:
			print "skipping ", fileNameWithPath
			return
		if image == None: return
		if image.hasThumb() == False:
			image.generateThumb()
		
		if self.__imageListener: self.__imageListener.onImageCreated(image, self.__imageListenerObject)
		print "crated", image
		return image
