from unittest import *
from os import *

from dim.dimlib.Library import *
from dim.dimlib.EXIFView import *


class EXIFViewTest(TestCase):
	
	def testGetExifFormatted(self):
		view = EXIFView("/home/anssi/pics/IMG_2206.JPG")
		exif = view.getExifFormatted()
		self.assertNotEqual(exif, None)

class ImageTest(TestCase):
	
	def setUp(self):
		[remove(path.join("dim/tests/thumbs", f)) for f in os.listdir("dim/tests/thumbs") if f != "CVS"]
		self.image = DimImage("dim/tests/images/test.jpg", "dim/tests/thumbs")

	def testCreateInvalid(self):
		try:
			image = DimImage("foobar.jpg", "foobar", True)
		except NoImageException:
			return
		self.fail("should have thrown NoImageException")

	def tearDown(self):
		[remove(path.join("dim/tests/thumbs", f)) for f in os.listdir("dim/tests/thumbs") if f != "CVS"]

	def testHasThumb(self):
		self.assertEqual(self.image.hasThumb(), False, "correctly recognizs that the image does not have a thumb")
	
	def testCreateThumb(self):
		self.image.generateThumb()
		self.assertNotEqual(self.image.hasThumb(), None, "thumbnail was created")

	def testGetEXIF(self):
		self.assertNotEqual(self.image.getEXIF(), None, "EXIF was returned")
	
	def testGetFileName(self):
		self.assertEqual(self.image.getFileName(), "dim/tests/images/test.jpg", "getFileName works")
	
	def testGetFileBasename(self):
		self.assertEqual(self.image.getFileBasename(), "test.jpg", "getFileBasename works")

class CatalogTest(TestCase):

	def setUp(self):
		self.catalog = Catalog("test")
		self.image = DimImage("dim/tests/images/test.jpg", "dim/tests/thumbs")
	
	def testAdd(self):
		self.catalog.add(self.image)
		self.assertEqual(self.catalog.hasImage(self.image), True, "added image is in the catalog")

		self.catalog.add(self.image)
		images = self.catalog.getImages()
		self.assertEqual(images.count(self.image), 1, "the catalog will not add the same image twice")
	
	def testRemove(self):
		self.catalog.add(self.image)
		self.catalog.remove(self.image)
		self.assertEqual(self.catalog.hasImage(self.image), False, "image was removed")
	
	def testGetImages(self):
		self.catalog.add(self.image)
		image2 = DimImage("dim/tests/images/test.jpg", "dim/tests/thumbs")
		image3 = DimImage("dim/tests/images/test2.jpg", "dim/tests/thumbs")
		self.catalog.add(image2)
		self.catalog.add(image3)

		images = self.catalog.getImages()
		self.assertEqual(self.image in images, True, "returned image list contains added image")
		self.assertEqual(image2 in images, True, "returned image list contains added image (2)")
		self.assertEqual(image3 in images, True, "returned image list contains added image (3)")

		images.remove(self.image)
		self.assertEqual(self.catalog.hasImage(self.image), True, "a copy of the catalog's list was returned")
		
	def testHasImage(self):
		self.catalog.add(self.image)
		self.assertEqual(self.catalog.hasImage(DimImage(self.image.getFileName(), "dim/tests/thumbs")), False, "hasImage is based on object identity")
		self.assertEqual(self.catalog.hasImageWithFileName(self.image.getFileName()), True, "hasImageWithFileName works with image's file name")

class LibraryTest(TestCase):
	
	def setUp(self):
		self.library = Library()
	
	def testGetCatalog(self):
		catalog = self.library.addCatalog("test")
		catalog2 = self.library.getCatalog("test")
		self.assertEqual(catalog, catalog2, "get catalog returns correctly with catalog's name")

	def testAddCatalog(self):
		catalog = self.library.addCatalog("test")
		self.assertEqual(self.library.hasCatalog(catalog), True, "added catalog is in the library")

		self.library.addCatalog("test")
		catalogs = self.library.getCatalogs()
		self.assertEqual(len(catalogs), 1, "will not add the same catalog twice")
	
	def testRemoveCatalog(self):
		catalog = self.library.addCatalog("test")
		self.library.removeCatalog(catalog)
		self.assertEqual(self.library.hasCatalog(catalog), False, "catalog was removed")
		
class IndexTest(TestCase):
	def setUp(self):
		self.index = Index("dim/tests/thumbs")
		self.index.setImageListener(self)

	def tearDown(self):
		[remove(path.join("dim/tests/thumbs", f)) for f in os.listdir("dim/tests/thumbs") if f != "CVS"]
		self.imageCreateEventReceived = False
		self.folderLoadedEventReceived = False
	
	def testCreateFolderImages(self):
		list = self.index.createFolderImages("dim/tests/images")
		self.assertNotEqual(list, None, "returned a list of images")
		self.assertNotEqual(len(list), 0, "list is not empty")
		self.assertEqual(self.imageCreateEventReceived, True, "received events for image creation")
		self.assertEqual(self.folderLoadedEventReceived, True, "received events for loaded folder")

		list = self.index.getFolderImages("dim/tests/images")
		self.assertNotEqual(list, None, "getFolderImages does not return None")
		self.assertNotEqual(len(list), 0, "getFolderImages returns a nonempty list")
		self.assertEqual(isinstance(list[0], DimImage), True, "getFolderImages returns a list of DimImages")
	
	def onImageCreated(self, image, object):
		print "received event for image ", image
		self.imageCreateEventReceived = True
	
	def onImageFolderLoaded(self, list, object):
		print "received event for folder loaded", list
		self.folderLoadedEventReceived = True

if __name__ == '__main__':
    main()